<?php

declare(strict_types=1);

namespace Dedi\SyliusImageOptimizerPlugin\Resolver;

use Liip\ImagineBundle\Imagine\Cache\CacheManager;
use Liip\ImagineBundle\Imagine\Data\DataManager;
use Liip\ImagineBundle\Imagine\Filter\FilterManager;
use Psr\Log\LoggerInterface;

class ImageResolverOnTheFly implements ImageResolverInterface
{
    private CacheManager $cacheManager;
    private DataManager $dataManager;
    private FilterManager $filterManager;
    private string $defaultFilterName;
    private array $defaultFilterOptions;
    private string $defaultFilterSizeKeyname;
    private LoggerInterface $logger;

    public function __construct(
        LoggerInterface $logger,
        CacheManager $cacheManager,
        DataManager $dataManager,
        FilterManager $filterManager,
        string $defaultFilterName,
        array $defaultFilterOptions,
        string $defaultFilterSizeKeyname
    ) {
        $this->cacheManager = $cacheManager;
        $this->dataManager = $dataManager;
        $this->filterManager = $filterManager;
        $this->defaultFilterName = $defaultFilterName;
        $this->defaultFilterOptions = $defaultFilterOptions;
        $this->defaultFilterSizeKeyname = $defaultFilterSizeKeyname;
        $this->logger = $logger;
    }

    public function resolve(string $filterName, string $imageName, array $options): string
    {
        $dirname = pathinfo($imageName, \PATHINFO_DIRNAME);
        $filename = pathinfo($imageName, \PATHINFO_FILENAME);

        $filtersOption = [];
        if (array_key_exists('filters', $options)) {
            $filtersOption[$options['filters']['name']] = $options['filters']['options'];
            $fullfileName = sprintf('%s_%s%s', $filename, $options['filters']['name'], $this->getOptionValue($options['filters']['options']));
        } else {
            $filtersOption[$this->defaultFilterName] = array_merge([
                $this->defaultFilterSizeKeyname => [$options['width'], $options['height']],
            ], $this->defaultFilterOptions);

            $fullfileName = sprintf('%s_%s%s_%d_%d', $filename, $this->defaultFilterName, $this->getOptionValue($this->defaultFilterOptions), $options['width'], $options['height']);
        }

        $cacheName = $dirname !== '.' ? $dirname . \DIRECTORY_SEPARATOR . $fullfileName : $fullfileName;

        try {
            if (!$this->cacheManager->isStored($cacheName, $filterName)) {
                $binary = $this->dataManager->find($filterName, $imageName);

                $filteredBinary = $this->filterManager->applyFilter($binary, $filterName, [
                    'filters' => $filtersOption,
                ]);

                $this->cacheManager->store($filteredBinary, $cacheName, $filterName);
            }

            return $this->cacheManager->resolve($cacheName, $filterName);
        } catch (\Exception $e) {
            $this->logger->error($e->getMessage());

            return $imageName;
        }
    }

    /**
     * Concate values of array recursively separate with underscore
     */
    private function getOptionValue(array $options): string
    {
        $valueName = '';

        foreach ($options as $value) {
            if (is_array($value)) {
                $valueName .= $this->getOptionValue($value);
            } else {
                $valueName .= '_' . (is_bool($value) ? (int) (string) $value : $value);
            }
        }

        return $valueName;
    }
}
